<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Entity;

use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\DTO\ReferralDTO;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Model\InviteeModelInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Model\InviterModelInterface;
use Doctrine\ORM\Mapping as ORM;
use Sylius\Component\Resource\Model\TimestampableTrait;

/**
 * @ORM\Entity(repositoryClass="Dedi\SyliusCustomerReferralPlugin\Infrastructure\Repository\ReferralRepository")
 * @ORM\Table(name="dedi_customer_referrral_referral")
 */
class Referral implements ReferralInterface
{
    use TimestampableTrait;

    /**
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="IDENTITY")
     *
     * @var int
     */
    protected $id;

    /**
     * @ORM\ManyToOne(targetEntity="Dedi\SyliusCustomerReferralPlugin\Entity\InviterInterface", inversedBy="referrals")
     *
     * @var InviterModelInterface
     */
    protected $inviter;

    /**
     * @ORM\Column(type="string", unique=true)
     *
     * @var string
     */
    protected $inviteeEmail;

    /**
     * @ORM\OneToOne(targetEntity="Dedi\SyliusCustomerReferralPlugin\Entity\InviteeInterface", inversedBy="referral")
     *
     * @var InviteeModelInterface
     */
    protected $invitee;

    public function getId()
    {
        return $this->id;
    }

    public function __construct()
    {
    }

    /**
     * @param ReferralDTO $referralDTO
     *
     * @return self
     */
    public static function createFromReferralDTO(
        $referralDTO
    ) {
        $entity = new self();
        $entity->inviteeEmail = $referralDTO->getInviteeEmail();
        $entity->inviter = $referralDTO->getInviter();

        return $entity;
    }

    public function getInviter(): InviterModelInterface
    {
        return $this->inviter;
    }

    public function getInvitee(): ?InviteeModelInterface
    {
        return $this->invitee;
    }

    public function getInviterFullName()
    {
        $customer = $this->inviter->getCustomer();

        return sprintf(
            '%s %s',
            $customer->getFirstName(),
            $customer->getLastName(),
        );
    }

    public function getInviteeFullName()
    {
        $customer = $this->invitee->getCustomer();

        return sprintf(
            '%s %s',
            $customer->getFirstName(),
            $customer->getLastName(),
        );
    }

    public function getInviteeEmail(): string
    {
        return $this->inviteeEmail;
    }

    public function setInvitee(InviteeModelInterface $invitee): void
    {
        $this->invitee = $invitee;
    }

    public function getCode(): string
    {
        return base64_encode(sprintf('%d:%s', $this->id, $this->inviteeEmail));
    }
}
